# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2020-10-06 12:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aimmo', '0020_add_info_to_worksheet'),
    ]

    operations = [
        migrations.AddField(
            model_name='worksheet',
            name='student_pdf_name',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name='worksheet',
            name='teacher_pdf_name',
            field=models.CharField(blank=True, max_length=255),
        ),
    ]
