{% load static %}
{% load app_tags %}
<!DOCTYPE html>

<head>
  <!-- Google Tag Manager -->
   <!-- End Google Tag Manager -->
  <title>Kurono</title>
  <link
    rel="shortcut icon"
    href="{% static 'favicon.ico' %}"
    type="image/x-icon"
  />
  <style type="text/css">
    body {
      margin: 0;
      padding: 0;
      overflow: hidden;
    }
  </style>
  <script>
    var appData = {
      urlForAimmoDashboard: "{% url_for_aimmo_dashboard %}"
    };
  </script>
</head>

<body>
  <!-- Google Tag Manager (noscript) -->
   <!-- End Google Tag Manager (noscript) -->
  <div id="root"></div>
  <script src="{% static 'react/src.3eef0766.js' %}"></script>
</body>
