# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2020-06-29 10:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aimmo', '0013_alter_game_can_play'),
    ]

    operations = [
        migrations.CreateModel(
            name='Worksheet',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('era', models.PositiveSmallIntegerField(choices=[(1, 'future'), (2, 'ancient'), (3, 'modern day'), (4, 'prehistoric'), (5, 'broken future')], default=1)),
                ('starter_code', models.TextField()),
            ],
        ),
    ]
